# AUTOGENERATED! DO NOT EDIT! File to edit: query.ipynb (unless otherwise specified).

__all__ = ['Querier', 'allQuery', 'ProductsFromList', 'productsFromList', 'singleProductQuery']

# Cell
import pandas as pd
from datetime import datetime
from nicHelper.wrappers import add_class_method, add_method, add_static_method
from nicHelper.dictUtil import printDict
from dataclasses_json import dataclass_json, Undefined, CatchAll
from dataclasses import dataclass
import logging
from typing import List
from s3bz.s3bz import S3
import pickle, json , boto3, zlib, os

# Cell
try:
  INVENTORY_BUCKET_NAME = os.environ['INVENTORY_BUCKET_NAME']
except Exception as e:
  print(f'missing environment variable {e} in query NB')
  INVENTORY_BUCKET_NAME = None

# Cell
class Querier:

  @staticmethod
  def validateInputQuery(keys: list, input_:dict):
    '''
      check if input query contains the valid key
      data should have the following structure
      key is a list of keys to check

      ib_prcode: String?
      ib_brcode: String?

      option, one of or both of the ib_procde must be present
    '''
    for key in keys:
      if key not in input_.keys():
        raise ValueError(f"key {key} is missing from the input")
      if not type(input_.get(key))==int:
        raise ValueError(f'key is not convertable to in {input_.get(key)}')
    return True

# Cell
@add_class_method(Querier)
def allQuery(cls, key = 'allData', bucket = os.environ.get('INVENTORY_BUCKET_NAME'), **kwargs):
  print(bucket)
  result = S3.presign(key, bucket = bucket, checkExist=False,  **kwargs)
  return result

# Cell
@dataclass_json
@dataclass
class ProductsFromList:
  cprcodes: List[str]


# Cell
import sys
@add_class_method(Querier)
def productsFromList(cls,cprcodes:List[str])->pd.DataFrame:
  db = cls.loadFromCache()
  print(f'db shape is {db.shape}')
  try:
    products = db[db['cprcode'].isin(cprcodes)]
  except:
    print(db.head())
    raise Exception(f'{db.head()} is not a valid db')
  return products

# Cell
@add_class_method(Querier)
def singleProductQuery(cls, input_)->pd.Series:
#   if not cls.validateInputQuery(['iprcode'] , input_): return f"error input {input_}"
  iprcode = int(input_['iprcode'])
  if (result:=next(cls.query(iprcode),None)): return result.toSeries()
  else: raise Exception('product not found')


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Helpers": "Helpers.ipynb",
         "notify": "Helpers.ipynb",
         "keys": "Helpers.ipynb",
         "scanDb": "Helpers.ipynb",
         "toDf": "Helpers.ipynb",
         "fromSeries": "Helpers.ipynb",
         "fromDf": "Helpers.ipynb",
         "fromDict": "Helpers.ipynb",
         "toListDict": "Helpers.ipynb",
         "toSeries": "Helpers.ipynb",
         "setNoUpdate": "Helpers.ipynb",
         "setUpdate": "Helpers.ipynb",
         "DATABASE_TABLE_NAME": "database.ipynb",
         "INVENTORY_BUCKET_NAME": "database.ipynb",
         "INPUT_BUCKET_NAME": "database.ipynb",
         "REGION": "database.ipynb",
         "ACCESS_KEY_ID": "database.ipynb",
         "SECRET_ACCESS_KEY": "database.ipynb",
         "LINEKEY": "database.ipynb",
         "createIndex": "KeySchema.ipynb",
         "KeySchema": "KeySchema.ipynb",
         "DBHASHLOCATION": "database.ipynb",
         "DBCACHELOCATION": "database.ipynb",
         "ALLDATAKEY": "S3.ipynb",
         "DEFAULTKEYS": "S3.ipynb",
         "S3Cache": "S3.ipynb",
         "loadFromCache": "S3.ipynb",
         "saveRemoteCache": "S3.ipynb",
         "resetS3Cache": "S3.ipynb",
         "BRANCH": "Update.ipynb",
         "VALUEUPDATESCHEMAURL": "Update.ipynb",
         "Updater": "Update.ipynb",
         "updateWithDict": "Update.ipynb",
         "ValueUpdate2": "Update.ipynb",
         "saveBatchToDynamodb": "Update.ipynb",
         "saveBatchToDynamodb2": "Update.ipynb",
         "valueUpdate2": "Update.ipynb",
         "INTCOLS": "database.ipynb",
         "ProductDatabase": "database.ipynb",
         "cacheDb": "database.ipynb",
         "lambdaDumpToS3": "database.ipynb",
         "Product": "database.ipynb",
         "ValueUpdate": "database.ipynb",
         "chunks": "database.ipynb",
         "lambdaUpdateProduct": "database.ipynb",
         "updateS3Input": "database.ipynb",
         "lambdaUpdateS3": "database.ipynb",
         "ProductsFromList": "query.ipynb",
         "lambdaProductsFromList": "database.ipynb",
         "lambdaSingleQuery": "database.ipynb",
         "lambdaAllQuery": "database.ipynb",
         "lambdaAllQueryFeather": "database.ipynb",
         "Querier": "query.ipynb",
         "allQuery": "query.ipynb",
         "productsFromList": "query.ipynb",
         "singleProductQuery": "query.ipynb"}

modules = ["helpers.py",
           "schema.py",
           "s3.py",
           "update.py",
           "database.py",
           "query.py"]

doc_url = "https://thanakijwanavit.github.io/villaProductDatabase/"

git_url = "https://github.com/thanakijwanavit/villaProductDatabase/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: Helpers.ipynb (unless otherwise specified).

__all__ = ['Helpers', 'notify', 'keys', 'scanDb', 'toDf', 'fromSeries', 'fromDf', 'fromDict', 'toListDict', 'toSeries',
           'setNoUpdate', 'setUpdate']

# Cell
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from nicHelper.dictUtil import stripDict
from linesdk.linesdk import Line
from typing import List
import pandas as pd
import os, logging



# Cell
class Helpers:
  pass

# Cell
@add_static_method(Helpers)
def notify(message):
  line = Line(os.environ.get('LINEKEY'))
  r = line.sendNotify(message)
  if r.status_code != 200:
    print(r.json())
    return False
  return True

# Cell
@add_class_method(Helpers)
def keys(cls):
  keys = list(vars(cls)['_attributes'].keys())
  return keys

# Cell
@add_class_method(Helpers)
def scanDb(cls, limit=100):
  return list(cls.scan())

# Cell
@add_class_method(Helpers)
def toDf(cls, products:List[Helpers])->pd.DataFrame:
  df = pd.DataFrame([item.data for item in products])
#   df = pd.DataFrame()
#   for product in products:
#     productSeries = product.toSeries()
#     productSeries = productSeries.rename(product.cprcode)
#     df = df.append(productSeries)
  return df

# Cell
@add_class_method(Helpers)
def fromSeries(cls, data:pd.Series)->Helpers:
  return cls.fromDict(data.to_dict())


# Cell
@add_class_method(Helpers)
def fromDf(cls, data:pd.DataFrame)->List[Helpers]:
  return data.apply(lambda x: cls.fromSeries(x), axis=1).to_list()


# Cell
@add_class_method(Helpers)
def fromDict(cls, dictInput):
  logging.debug(dictInput)
  dictInput = stripDict(dictInput)
  #### extract the keys
  filteredInput = {k:v for k,v in dictInput.items() if k in cls.keys()}
  #### save whole object to dictInput
  filteredInput['data'] = dictInput
  logging.debug(filteredInput)
  return cls(**filteredInput)

# Cell
@add_class_method(Helpers)
def toListDict(cls, items:List[Helpers]):
  return [item.data for item in items]

# Cell
@add_method(Helpers)
def toSeries(self):
  return pd.Series(self.data)

# Cell
@add_method(Helpers)
def setNoUpdate(self, batch = None):
  self.needsUpdate = self.FALSE
  if batch:
    return batch.save(self)
  else:
    return self.save()

@add_method(Helpers)
def setUpdate(self, batch=None):
  self.needsUpdate = self.TRUE
  if batch:
    return batch.save(self)
  else:
    return self.save()


# AUTOGENERATED! DO NOT EDIT! File to edit: Update.ipynb (unless otherwise specified).

__all__ = ['DBHASHLOCATION', 'DBCACHELOCATION', 'DATABASE_TABLE_NAME', 'INVENTORY_BUCKET_NAME', 'INPUT_BUCKET_NAME',
           'REGION', 'ACCESS_KEY_ID', 'SECRET_ACCESS_KEY', 'LINEKEY', 'BRANCH', 'VALUEUPDATESCHEMAURL', 'Updater',
           'updateWithDict', 'ValueUpdate2', 'saveBatchToDynamodb', 'saveBatchToDynamodb2', 'valueUpdate2']

# Cell
from s3bz.s3bz import S3
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from nicHelper.dictUtil import stripDict, printDict, hashDict, saveStringToFile, loadStringFromFile, saveDictToFile, loadDictFromFile
from nicHelper.exception import errorString
from dict_hash import dict_hash, sha256
from base64 import b64encode, b64decode
from dataclasses_json import dataclass_json, Undefined, CatchAll
from dataclasses_jsonschema import JsonSchemaMixin
from dataclasses import dataclass
from jsonschema import validate
from typing import List
from datetime import datetime
from nicHelper.schema import validateUrl
import pandas as pd
import os, logging, json

# Cell
import os
DBHASHLOCATION = '/tmp/database.hash'
DBCACHELOCATION = '/tmp/database.cache'
DATABASE_TABLE_NAME = os.environ.get('DATABASE_TABLE_NAME')
INVENTORY_BUCKET_NAME = os.environ.get('INVENTORY_BUCKET_NAME')
INPUT_BUCKET_NAME = os.environ.get('INPUT_BUCKET_NAME')
REGION = os.environ.get('REGION') or 'ap-southeast-1'
ACCESS_KEY_ID = os.environ.get('USER') or None
SECRET_ACCESS_KEY = os.environ.get('PW') or None
LINEKEY= os.environ.get('LINEKEY')
BRANCH = os.environ.get('BRANCH')
VALUEUPDATESCHEMAURL = f'https://raw.githubusercontent.com/thanakijwanavit/villaMasterSchema/{BRANCH}/product/product.yaml'

try:
  DAX_ENDPOINT = os.environ['DAX_ENDPOINT']
  print(DAX_ENDPOINT)
except KeyError as e:
  print(f'dax endpoint missing {e}')

# Cell
class Updater:
  pass

# Cell
@add_class_method(Updater)
def updateWithDict(cls, originalObject:Updater, inputDict:dict ):
  data = originalObject.data
  data.update(inputDict)
  return cls.fromDict(data)

# Cell
class ValueUpdate2:
   #### divide work into chunks
  @staticmethod
  def chunks(l, n): return [l[x: x+n] for x in range(0, len(l), n)]
  ##### plot time elapsed for debugging
  @staticmethod
  def getTimeElasped(t0): return (datetime.now()-t0).total_seconds() * 1000

  t0 = datetime.now()

  ### validate input
  @staticmethod
  def validateInput(input_):
    return validateUrl(VALUEUPDATESCHEMAURL,
                       input_, format_='yaml')

# Cell
@add_static_method(ValueUpdate2)
def saveBatchToDynamodb(productBatch:list, dbClass:Updater, db:pd.DataFrame, itemsUpdated:dict):
  def checkIfInDb(db:pd.DataFrame, iprcode:int, cprcode:int)->dbClass:
    '''check if product is in the database, if not, create an empty class with the product code'''
    if 'cprcode' in db.columns:
      incumbentDb = db[db['cprcode']==cprcode] ## filter for the cprcode
      if incumbentDb.empty:
        incumbentSeries = pd.Series() ## if empty, create an empty series
      else:
        incumbentSeries = incumbentDb.iloc[0] ## if not empty, pull the series

      #### create object from series or new object if series is empty ####
      if incumbentSeries.any:  incumbentItem = dbClass.fromSeries(incumbentSeries)
      else: incumbentItem = dbClass.fromDict({'iprcode': iprcode, 'cprcode': cprcode})

    #### if cprcode doesnt exist, create empty object ####
    else: incumbentItem = dbClass.fromDict({'iprcode': iprcode, 'cprcode': cprcode})
    return incumbentItem

  ###### main ######
  with dbClass.batch_write() as batch:
    # loop through each product
    for product in productBatch:
      iprcode = product['iprcode']
      cprcode = product['cprcode']
      incumbentItem = checkIfInDb(db, iprcode, cprcode) ### check if product in db or return empty obj

      ##### make a copy of original data
      originalData = incumbentItem.data.copy()
      ###### update data
      updatedData = dbClass.updateWithDict(incumbentItem, product)

      logging.info(f'incumbentBr is {incumbentItem.iprcode}\n, prcode is {iprcode}')

      # check for difference
      try:
        if updatedData.data != originalData:
          logging.info(f'product {iprcode} has changed from \n{originalData} \n{updatedData.data}')
          batch.save(updatedData)
          itemsUpdated['success'] += 1
        else:
          logging.info(f'no change for {iprcode}')
          itemsUpdated['skipped'] += 1
      except Exception as e:
        itemsUpdated['failure'] += 1
        itemsUpdated['failureMessage'].append(e)

    # log time taken
#     itemsUpdated['timetaken(ms)'] = (datetime.now()- t0).total_seconds()*1000

# Cell
@add_static_method(ValueUpdate2)
def saveBatchToDynamodb2(productBatch:list, dbClass:Updater, db:pd.DataFrame, itemsUpdated:dict):
  def checkIfInDb(db:pd.DataFrame, iprcode:int, cprcode:int)->dbClass:
    '''check if product is in the database, if not, create an empty class with the product code'''
    incumbentItem =  next(dbClass.query(hash_key = iprcode, range_key_condition=dbClass.cprcode == cprcode), dbClass(iprcode=iprcode, cprcode=cprcode, data = {}))
    return incumbentItem

  ###### main ######
  with dbClass.batch_write() as batch:
    # loop through each product
    for product in productBatch:
      iprcode = product['iprcode']
      cprcode = product['cprcode']
      incumbentItem = checkIfInDb(db, iprcode, cprcode) ### check if product in db or return empty obj

      ##### make a copy of original data
      originalData = incumbentItem.data.copy()
      ###### update data
      updatedData = dbClass.updateWithDict(incumbentItem, product)

      # check for difference
      try:
        if updatedData.data != originalData:
          logging.info(f'product {iprcode} has changed from \n{originalData} \n{updatedData.data}')
          batch.save(updatedData)
          itemsUpdated['success'] += 1
        else:
          logging.info(f'no change for {iprcode}')
          itemsUpdated['skipped'] += 1

      except Exception as e:
        itemsUpdated['failure'] += 1
        itemsUpdated['failureMessage'].append(e)



# Cell
@add_class_method(Updater)
def valueUpdate2(cls, inputs:List[dict]):
    '''
      check for difference and batch update the changes in product data
    '''
    #### setup #####
    itemsUpdated = {'success':0, 'failure': 0, 'skipped': 0 ,'failureMessage':[], 'timetaken(ms)': 0}
    t0 = datetime.now()

    ##### main #####

    ######validate input ###########
    products = ValueUpdate2.validateInput(inputs).items
    print(f'products are {products[0]}')
    print(f'time taken for validation  {ValueUpdate2.getTimeElasped(t0)} ms')

    ######dividing input into batch of 500###########
    productBatches = ValueUpdate2.chunks(products, 500)
    print(f'divided into chunks {ValueUpdate2.getTimeElasped(t0)} ms')

    ######try to load from cache########
    db = cls.loadFromCache().fillna('none')
    print(f'get all from s3 {ValueUpdate2.getTimeElasped(t0)} ms')

    ###### save to dynamodb ########
    for productBatch in productBatches:
      ValueUpdate2.saveBatchToDynamodb2(productBatch, cls, db, itemsUpdated)
    itemsUpdated['timetaken(ms)'] = (datetime.now()- t0).total_seconds()*1000
    return itemsUpdated



# AUTOGENERATED! DO NOT EDIT! File to edit: database.ipynb (unless otherwise specified).

__all__ = ['DATABASE_TABLE_NAME', 'INVENTORY_BUCKET_NAME', 'INPUT_BUCKET_NAME', 'REGION', 'ACCESS_KEY_ID',
           'SECRET_ACCESS_KEY', 'LINEKEY', 'DBHASHLOCATION', 'DBCACHELOCATION', 'INTCOLS', 'ProductDatabase', 'cacheDb',
           'lambdaDumpToS3', 'Product', 'ValueUpdate', 'chunks', 'lambdaUpdateProduct', 'updateS3Input',
           'lambdaUpdateS3', 'ProductsFromList', 'lambdaProductsFromList', 'lambdaSingleQuery', 'lambdaAllQuery',
           'lambdaAllQueryFeather']

# Cell
import pandas as pd
from datetime import datetime
from pynamodb.models import Model
from pynamodb.attributes import UnicodeAttribute, NumberAttribute, JSONAttribute, BooleanAttribute, BinaryAttribute
from pynamodb.indexes import GlobalSecondaryIndex, AllProjection
from botocore.config import Config
from s3bz.s3bz import S3
from pprint import pprint
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from nicHelper.dictUtil import stripDict, printDict
from nicHelper.exception import errorString
from nicHelper import pdUtils
from awsSchema.apigateway import Response, Event
from dataclasses_json import dataclass_json, Undefined, CatchAll
from dataclasses import dataclass
from typing import List
from .query import Querier
from .helpers import Helpers
from .s3 import S3Cache
from .schema import KeySchema, createIndex
from .update import Updater
from requests import post

import pickle, json, boto3, bz2, requests, validators, os, logging, traceback, zlib

# Cell
import os

DATABASE_TABLE_NAME = os.environ.get('DATABASE_TABLE_NAME')
INVENTORY_BUCKET_NAME = os.environ.get('INVENTORY_BUCKET_NAME')
INPUT_BUCKET_NAME = os.environ.get('INPUT_BUCKET_NAME')
REGION = os.environ.get('REGION') or 'ap-southeast-1'
ACCESS_KEY_ID = os.environ.get('USER') or None
SECRET_ACCESS_KEY = os.environ.get('PW') or None
LINEKEY= os.environ.get('LINEKEY')
DBHASHLOCATION = '/mnt/efs/database.hash'
DBCACHELOCATION = '/mnt/efs/database.cache'
INTCOLS = json.loads(os.environ.get('INTCOLS') or '{}')

try:
  DAX_ENDPOINT = os.environ['DAX_ENDPOINT']
except KeyError as e:
  DAX_ENDPOINT = None
  print(f'dax endpoint missing {e}')

print(DAX_ENDPOINT)

# Cell
# dont forget to import dependent classes from the relevant notebooks
class ProductDatabase( Querier, Helpers, KeySchema, S3Cache, Updater):
  class Meta:
    aws_access_key_id = ACCESS_KEY_ID
    aws_secret_access_key = SECRET_ACCESS_KEY
    table_name = DATABASE_TABLE_NAME
    region = REGION
    billing_mode='PAY_PER_REQUEST'
    dax_read_endpoints = [DAX_ENDPOINT] if DAX_ENDPOINT else None
    dax_write_endpoints = [DAX_ENDPOINT] if DAX_ENDPOINT else None


  TRUE = 'Y'
  FALSE = 'N'

  # indexes
  needsUpdateIndex = createIndex('needsUpdate','sellingPrice')
  cprcodeIndex = createIndex('cprcode', 'sellingPrice')
  oprcodeIndex = createIndex('oprcode', 'sellingPrice')
  pr_dpcodeIndex = createIndex('pr_dpcode', 'sellingPrice')
  pr_barcodeIndex = createIndex('pr_barcode', 'sellingPrice')
  pr_barcode2Index = createIndex('pr_barcode2', 'sellingPrice')
  pr_suref3Index = createIndex('pr_suref3', 'sellingPrice')
  pr_sa_methodIndex = createIndex('pr_sa_method', 'sellingPrice')



  def __repr__(self):
    return self.returnKW(self.data)
  def syncIndex(self):
    keys = {k:v for k,v in self.data if k in []}


  @staticmethod
  def returnKW(inputDict):
    outputStr = 'ProductDatabase Object\n'
    for k,v in inputDict.items():
      outputStr += f'{k} {v}\n'
    return outputStr
  @staticmethod
  def convertIntCols(products:List[dict])->List[dict]:
    convertedProducts = [{k:int(float(v)) if k in INTCOLS else v for k,v in product.items()}for product in products]
    return convertedProducts





# Cell
@add_class_method(ProductDatabase)
def cacheDb(cls, bucketName = INVENTORY_BUCKET_NAME, key = 'allData', limit=100, **kwargs):
  '''cache db to s3 and local efs'''
  db:pd.DataFrame = cls.loadFromCache(key=key, localCache=DBCACHELOCATION,
                                      localHash=DBHASHLOCATION, bucket=bucketName)
  logging.debug(f'origin item is shape{db.shape}')
  #### get change
  print('quering for changes')
  changes = list(cls.needsUpdateIndex.query(cls.TRUE, limit=limit))
  print('convert to df')
  changesDf = cls.toDf(changes)
  changesDf.set_index('cprcode', inplace=True)
  print(changesDf.shape)
#   db.set_index('cprcode', inplace=True)
  updatedDb = db.append(changesDf).drop_duplicates('cprcode', keep='last')
#   updatedDb.reset_index(inplace=True)
  cls.saveRemoteCache(updatedDb)
  print('saving to remote cache')
  with cls.batch_write() as batch:
    for item in changes:
      item.setNoUpdate(batch=batch)
  cls.notify(f'db shape is {db.shape}')
  return updatedDb

# Cell
def lambdaDumpToS3(event, _):
  result = ProductDatabase.cacheDb(limit = 500)
  lastItem = result.iloc[0].to_json()
  return Response.returnSuccess({'result': json.loads(lastItem)})
#   print(result.fillna(''))
#   return Response.getReturn(body = {'result': result.fillna('').iloc[0].to_dict()})

# Cell
@dataclass_json(undefined=Undefined.INCLUDE)
@dataclass
class Product:
  iprcode: int
  cprcode: int
  data: CatchAll
@dataclass_json
@dataclass
class ValueUpdate:
  items: List[Product]

# Cell
def chunks(l, n): return [l[x: x+n] for x in range(0, len(l), n)]

# Cell
def lambdaUpdateProduct (event, _):
  products = Event.parseBody(event)['products']
  convertedProducts = ProductDatabase.convertIntCols(products)
  result = ProductDatabase.valueUpdate2({'items':convertedProducts})
  return Response.getReturn(body = result)

# Cell
@add_class_method(ProductDatabase)
def updateS3Input(cls, inputBucketName:str = INPUT_BUCKET_NAME, key:str = '', **kwargs):
  products = S3.load(key=key, bucket = inputBucketName,  **kwargs)
  updateResult = cls.valueUpdate2({'items':cls.convertIntCols(products)})
  return updateResult


# Cell
def lambdaUpdateS3(event, _):
  inputKeyName = Event.from_dict(event).key()
  try:
    updateResult = ProductDatabase.updateS3Input(
      inputBucketName=INPUT_BUCKET_NAME, key= inputKeyName)
  except:
    ProductDatabase.notify(f'error updating with s3 {errorString()}')
    return Response.returnError(errorString())



  ProductDatabase.notify(f'success update {updateResult}')
  return Response.getReturn(body = updateResult)

# Cell
@dataclass_json
@dataclass
class ProductsFromList:
  iprcodes: List[str]

# Cell
def lambdaProductsFromList(event, *args):
  productsFromList = Event.parseDataClass(ProductsFromList,event)
  result:pd.DataFrame = ProductDatabase.productsFromList(productsFromList.iprcodes)
  results:List[ProductDatabase] = ProductDatabase.fromDf(result)
  resultDicts:List[dict] = ProductDatabase.toListDict(results)
  return Response.returnSuccess(resultDicts)

# Cell
def lambdaSingleQuery(event, _):
  key, value = Event.from_dict(event).firstKey()
  try:
    result = ProductDatabase.singleProductQuery({key:value}).to_dict()
  except Exception as e:
    return Response.returnError(f'{e}')
  return Response.returnSuccess(body = result)

# Cell
def lambdaAllQuery(event, *args):
  url = ProductDatabase.allQuery(bucket = INVENTORY_BUCKET_NAME, key='allData-json.zl')
  return Response.getReturn(body = {'url': url})

# Cell
def lambdaAllQueryFeather(event, *args):
  key = 'allData'
  bucket = INVENTORY_BUCKET_NAME
  url = ProductDatabase.allQuery(bucket = INVENTORY_BUCKET_NAME, key=key)
  hashCode = pdUtils.loadRemoteHash(key=key, bucket=bucket, useUrl = True)
  return Response.getReturn(body = {'url': url, 'hash': hashCode})




# Cell
import pandas as pd
from datetime import datetime
from pynamodb.models import Model
from pynamodb.attributes import UnicodeAttribute, NumberAttribute, JSONAttribute, BooleanAttribute, BinaryAttribute
from pynamodb.indexes import GlobalSecondaryIndex, AllProjection
from botocore.config import Config
from s3bz.s3bz import S3
from pprint import pprint
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from nicHelper.dictUtil import stripDict, printDict
from nicHelper.exception import errorString
from awsSchema.apigateway import Response, Event
from dataclasses_json import dataclass_json, Undefined, CatchAll
from dataclasses import dataclass
from typing import List
from .query import Querier
from requests import post
from linesdk.linesdk import Line

import pickle, json, boto3, bz2, requests, validators, os, logging, traceback

# Cell
import pandas as pd
from datetime import datetime
from pynamodb.models import Model
from pynamodb.attributes import UnicodeAttribute, NumberAttribute, JSONAttribute, BooleanAttribute, BinaryAttribute
from pynamodb.indexes import GlobalSecondaryIndex, AllProjection
from botocore.config import Config
from s3bz.s3bz import S3
from pprint import pprint
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from nicHelper.dictUtil import stripDict, printDict
from nicHelper.exception import errorString
from awsSchema.apigateway import Response, Event
from dataclasses_json import dataclass_json, Undefined, CatchAll
from dataclasses import dataclass
from typing import List
from .query import Querier
from requests import post
from linesdk.linesdk import Line

import pickle, json, boto3, bz2, requests, validators, os, logging, traceback


# AUTOGENERATED! DO NOT EDIT! File to edit: KeySchema.ipynb (unless otherwise specified).

__all__ = ['DATABASE_TABLE_NAME', 'INVENTORY_BUCKET_NAME', 'INPUT_BUCKET_NAME', 'REGION', 'ACCESS_KEY_ID',
           'SECRET_ACCESS_KEY', 'LINEKEY', 'createIndex', 'KeySchema']

# Cell
import os

DATABASE_TABLE_NAME = os.environ.get('DATABASE_TABLE_NAME')
INVENTORY_BUCKET_NAME = os.environ.get('INVENTORY_BUCKET_NAME')
INPUT_BUCKET_NAME = os.environ.get('INPUT_BUCKET_NAME')
REGION = os.environ.get('REGION') or 'ap-southeast-1'
ACCESS_KEY_ID = os.environ.get('USER') or None
SECRET_ACCESS_KEY = os.environ.get('PW') or None
LINEKEY= os.environ.get('LINEKEY')

try:
  DAX_ENDPOINT = os.environ['DAX_ENDPOINT']
except KeyError as e:
  DAX_ENDPOINT = None
  print(f'dax endpoint missing {e}')

print(DAX_ENDPOINT)

# Cell
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from pynamodb.attributes import UnicodeAttribute, NumberAttribute, JSONAttribute, BooleanAttribute, BinaryAttribute
from pynamodb.indexes import GlobalSecondaryIndex, AllProjection
from pynamodb.models import Model
from pprint import pformat

# Cell
def createIndex(name, rangeKeyName= None, indexName = None, HashKeyType = UnicodeAttribute, RangeKeyType = UnicodeAttribute):
  class ReturnSecondaryIndex(GlobalSecondaryIndex):
    class Meta:
      index_name = indexName or name
      projection = AllProjection()
      dax_read_endpoints = [DAX_ENDPOINT] if DAX_ENDPOINT else None
      dax_write_endpoints = [DAX_ENDPOINT] if DAX_ENDPOINT else None
      read_capacity_units = 1
      write_capacity_units = 1
  setattr(ReturnSecondaryIndex, name, HashKeyType(hash_key = True))
  if rangeKeyName:
    setattr(ReturnSecondaryIndex, rangeKeyName, RangeKeyType(range_key = True))
  return ReturnSecondaryIndex()

# Cell
class KeySchema(Model):
  iprcode = NumberAttribute(hash_key=True, default = 0)
  cprcode = NumberAttribute(default = 0, range_key = True)
  oprcode = NumberAttribute(default = 0)
  pr_dpcode = UnicodeAttribute(default = 'none')
  pr_barcode = UnicodeAttribute(default = 'none')
  pr_barcode2 = UnicodeAttribute(default = 'none')
  pr_sucode1 = UnicodeAttribute(default = 'none')
  pr_suref3 = UnicodeAttribute(default= 'none')
  pr_sa_method = UnicodeAttribute(default= 'none')
  sellingPrice = NumberAttribute(default = 0)
  lastUpdate = NumberAttribute( default = 0)
  needsUpdate = UnicodeAttribute(default = 'Y')
  data = JSONAttribute()

  # indexes
  needsUpdateIndex = createIndex('needsUpdate','sellingPrice')
  cprcodeIndex = createIndex('cprcode', 'sellingPrice')
  oprcodeIndex = createIndex('oprcode', 'sellingPrice')
  pr_dpcodeIndex = createIndex('pr_dpcode', 'sellingPrice')
  pr_barcodeIndex = createIndex('pr_barcode', 'sellingPrice')
  pr_barcode2Index = createIndex('pr_barcode2', 'sellingPrice')
  pr_suref3Index = createIndex('pr_suref3', 'sellingPrice')
  pr_sa_methodIndex = createIndex('pr_sa_method', 'sellingPrice')
  cprcodeOnlyIndex = createIndex('cprcode', indexName='cprcodeOnly')

  def __repr__(self):
    return pformat(vars(self)["attribute_values"])


# AUTOGENERATED! DO NOT EDIT! File to edit: S3.ipynb (unless otherwise specified).

__all__ = ['DBHASHLOCATION', 'DBCACHELOCATION', 'DATABASE_TABLE_NAME', 'INVENTORY_BUCKET_NAME', 'INPUT_BUCKET_NAME',
           'REGION', 'ACCESS_KEY_ID', 'SECRET_ACCESS_KEY', 'LINEKEY', 'ALLDATAKEY', 'DEFAULTKEYS', 'S3Cache',
           'loadFromCache', 'saveRemoteCache', 'resetS3Cache']

# Cell
from s3bz.s3bz import S3
from nicHelper.wrappers import add_method, add_class_method, add_static_method
from nicHelper.dictUtil import stripDict, printDict, hashDict, saveStringToFile, loadStringFromFile, saveDictToFile, loadDictFromFile
from nicHelper.exception import errorString
import nicHelper.pdUtils as pdUtils
from dict_hash import dict_hash, sha256
from base64 import b64encode, b64decode
from pandas.util import hash_pandas_object
from hashlib import sha1
import pandas as pd
import os, logging, zlib

# Cell
import os
DBHASHLOCATION = '/mnt/efs/database.hash'
DBCACHELOCATION = '/mnt/efs/database.cache'
DATABASE_TABLE_NAME = os.environ.get('DATABASE_TABLE_NAME')
INVENTORY_BUCKET_NAME = os.environ.get('INVENTORY_BUCKET_NAME')
INPUT_BUCKET_NAME = os.environ.get('INPUT_BUCKET_NAME')
REGION = os.environ.get('REGION') or 'ap-southeast-1'
ACCESS_KEY_ID = os.environ.get('USER') or None
SECRET_ACCESS_KEY = os.environ.get('PW') or None
LINEKEY= os.environ.get('LINEKEY')
ALLDATAKEY = 'allData'
DEFAULTKEYS = ['cprcode', 'iprcode', 'oprcode', 'ordertype', 'pr_abb', 'pr_active', 'pr_cgcode', 'pr_code', 'pr_dpcode', 'pr_engname', 'pr_ggcode', 'pr_market', 'pr_name', 'pr_puqty', 'pr_sa_method', 'pr_sucode1', 'pr_suref3', 'prtype', 'psqty', 'pstype']

try:
  DAX_ENDPOINT = os.environ['DAX_ENDPOINT']
  print(DAX_ENDPOINT)
except KeyError as e:
  print(f'dax endpoint missing {e}')


# Cell
class S3Cache:
  pass

# Cell
@add_class_method(S3Cache)
def loadFromCache(cls, key=ALLDATAKEY, localCache=DBCACHELOCATION,
                  localHash=DBHASHLOCATION, bucket=INVENTORY_BUCKET_NAME):
  ## check for local object and its hash
  if os.path.exists(localCache) and os.path.exists(localHash):
    try:
      localHash = pdUtils.loadLocalHash(path=localHash)
      logging.debug(f'localHash is {localHash}')
      remoteHash = pdUtils.loadRemoteHash(key=key, bucket=bucket)
      logging.debug(f'remoteHash is {remoteHash}')

      if localHash == remoteHash:
        print('data is still in sync, using local cache')
        db = pdUtils.loadLocalCache(path=localCache)
        return db
      else:
        print('remote hash is not the same, load remote cache')
    except Exception as e: print(f'local loading error{e}, loading remote hash')
  ### load from remote cache
  try:
    db = pdUtils.loadRemoteCache(key=key, bucket=bucket)
    pdUtils.saveLocalCache(db, path=localCache)
    pdUtils.saveLocalHash(db, path = localHash)
  except Exception as e:
    print(f'locding remtoe failed {e} returning blank df')
    db = pd.DataFrame(columns = DEFAULTKEYS)
  ### save to local cache
  return db

# Cell

@add_class_method(S3Cache)
def saveRemoteCache(cls ,db:pd.DataFrame, key= ALLDATAKEY,
                   bucket = INVENTORY_BUCKET_NAME, localCachePath=DBCACHELOCATION,
                   localHashPath=DBHASHLOCATION):
  if db.empty:
    db = pd.DataFrame([DUMMYDATA])
  db.columns = db.columns.astype(str)
  db = db.reset_index(drop=True)

#   print(db)
  pdUtils.saveRemoteCache(data=db, key= key,
                          bucket=bucket, localCachePath=localCachePath,
                          localHashPath=localHashPath)
  jsonDb = db.to_json(orient='split')
  zlibArc = zlib.compress(jsonDb.encode())
  tmpPath = '/tmp/zlibJsonCache.zl'
  with open(tmpPath, 'wb') as f:
    f.write(zlibArc)
  S3.saveFile(key=f'{key}-json.zl',path=tmpPath,bucket=bucket)



# Cell
@add_class_method(S3Cache)
def resetS3Cache(cls, bucketName= INVENTORY_BUCKET_NAME, key = 'allData', limit=10000, **kwargs):
  ''' upload changes to s3'''
  ###### get all data
  items:List[cls] = cls.scanDb(limit=limit)
  print('db is scanned')
  db:pd.DataFrame = cls.toDf(items)
  print(f'{db.shape} changes to update')
  cls.saveRemoteCache(db)
  return True